==> cbang_0.0.0.tar.gz: cbang-1.3.2/src/cbang/openssl/SSL.cpp <==
/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

                Copyright (c) 2003-2019, Cauldron Development LLC
                   Copyright (c) 2003-2017, Stanford University
                               All rights reserved.

         The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
       as published by the Free Software Foundation, either version 2.1 of
               the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                 Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
                 License along with the C! library.  If not, see
                         <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
           You may request written permission by emailing the authors.

                  For information regarding this software email:
                                 Joseph Coffland
                          joseph@cauldrondevelopment.com

\******************************************************************************/

#include "SSL.h"

#include "SecurityUtilities.h"
#include "BStream.h"
#include "Certificate.h"

#include <cbang/os/Mutex.h>
#include <cbang/os/Thread.h>

#include <cbang/log/Logger.h>

#ifdef HAVE_VALGRIND
#include <valgrind/memcheck.h>
#endif

#include <iostream>
#include <string.h>

// This avoids a conflict with OCSP_RESPONSE in wincrypt.h
#ifdef OCSP_RESPONSE
#undef OCSP_RESPONSE
#endif

#define OPENSSL_THREAD_DEFINES
#include <openssl/opensslconf.h>

#if !defined(OPENSSL_THREADS)
#error "OpenSSL must be compiled with thread support"
#endif

#include <openssl/crypto.h>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <openssl/opensslv.h>


using namespace std;
using namespace cb;


bool cb::SSL::initialized = false;
Mutex *cb::SSL::locks = 0;


cb::SSL::SSL(_SSL *ssl, bool deallocate) :
  ssl(ssl), renegotiateLimited(false), handshakes(0), deallocate(deallocate),
  state(PROCEED) {
  init();
}


cb::SSL::SSL(SSL_CTX *ctx, BIO *bio) :
  ssl(0), renegotiateLimited(false), handshakes(0), deallocate(true),
  state(PROCEED) {
  init();
  ssl = SSL_new(ctx);
  if (!ssl) THROW("Failed to create new SSL");
  if (bio) setBIO(bio);
}


cb::SSL::~SSL() {
  if (ssl && deallocate) {
    SSL_free(ssl);
    ssl = 0;
  }
}


void cb::SSL::setBIO(BIO *bio) {
  SSL_set_bio(ssl, bio, bio);
}


void cb::SSL::setCipherList(const string &list) {
  if (!SSL_set_cipher_list(ssl, list.c_str()))
    THROWS("Failed to set cipher list to: " << list);
}


string cb::SSL::getFullSSLErrorStr(int ret) const {
  string s = getSSLErrorStr(SSL_get_error(ssl, ret));
  if (peekError()) s += ", " + getErrorStr();
  return s;
}


bool cb::SSL::hasPeerCertificate() const {
  X509 *cert = SSL_get_peer_certificate(ssl);
  if (!cert) return false;
  X509_free(cert);
  return true;
}


void cb::SSL::verifyPeerCertificate() const {
  if (!hasPeerCertificate()) THROW("Peer did not present a certificate");

  if (SSL_get_verify_result(ssl) != X509_V_OK)
    THROW("Certificate does not verify");
}


SmartPointer<Certificate> cb::SSL::getPeerCertificate() const {
  if (SSL_get_verify_result(ssl) != X509_V_OK)
    THROW("Certificate does not verify");

  X509 *cert = SSL_get_peer_certificate(ssl);
  if (!cert) THROW("Peer did not present a certificate");

  return new Certificate(cert);
}


void cb::SSL::setTLSExtHostname(const string &hostname) {
  if (!SSL_set_tlsext_host_name(ssl, hostname.c_str()))
    THROWS("Failed to set TLS host name extension to '" << hostname << "'");
}


void cb::SSL::connect() {
  LOG_DEBUG(5, "cb::SSL::connect()");
  int ret = SSL_connect(ssl);

  if (ret == -1) {
    int err = SSL_get_error(ssl, ret);
    if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) {
      state = WANTS_CONNECT;
      return;
    }
  }

  state = PROCEED;

  if (ret != 1) THROWS("SSL connect failed: " << getFullSSLErrorStr(ret));
}


void cb::SSL::accept() {
  LOG_DEBUG(5, "cb::SSL::accept()");
  limitRenegotiation();
  int ret = SSL_accept(ssl);

  if (ret == -1) {
    int err = SSL_get_error(ssl, ret);
    if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) {
      state = WANTS_ACCEPT;
      return;
    }
  }

  state = PROCEED;

  // We can ignore WANT_READ etc. here because the Socket layer already
  // retries reads and writes up to the specified timeout.
  if (ret != 1) {
    string err = getFullSSLErrorStr(ret);
    LOG_DEBUG(5, "SSL accept failed: " << err);
    THROWS("SSL accept failed: " << err);
  }
}


void cb::SSL::shutdown() {
  SSL_shutdown(ssl);
  LOG_DEBUG(5, "cb::SSL::shutdown() " << getErrorStr());

  flushErrors(); // Ignore errors
}


int cb::SSL::read(char *data, unsigned size) {
  LOG_DEBUG(5, "cb::SSL::read(" << size << ')');

  checkHandshakes();
  if (!checkWants() || !size) return 0;

  int ret = SSL_read(ssl, data, size);
  if (ret <= 0) {
    // Detect End of Stream
    if (SSL_get_shutdown(ssl) == SSL_RECEIVED_SHUTDOWN) return -1;

    int err = SSL_get_error(ssl, ret);
    if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) return 0;

    string errMsg = getFullSSLErrorStr(ret);
    LOG_DEBUG(5, "cb::SSL::read() " << errMsg);
    THROWS("SSL read failed: " << errMsg);
  }

  LOG_DEBUG(5, "cb::SSL::read()=" << ret);

#ifdef VALGRIND_MAKE_MEM_DEFINED
  (void)VALGRIND_MAKE_MEM_DEFINED(data, ret);
#endif // VALGRIND_MAKE_MEM_DEFINED

  return (unsigned)ret;
}


unsigned cb::SSL::write(const char *data, unsigned size) {
  LOG_DEBUG(5, "cb::SSL::write(" << size << ')');

  checkHandshakes();
  if (!checkWants() || !size) return 0;

  int ret = SSL_write(ssl, data, size);
  if (ret <= 0) {
    int err = SSL_get_error(ssl, ret);
    if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE) return 0;
    THROWS("SSL write failed: " << getFullSSLErrorStr(ret));
  }

  LOG_DEBUG(5, "cb::SSL::write()=" << ret);
  return (unsigned)ret;
}


unsigned long cb::SSL::idCallback() {
  return (unsigned long)cb::Thread::self();
}


void cb::SSL::lockingCallback(int mode, int n, const char *file, int line) {
  if (mode & CRYPTO_LOCK) locks[n].lock();
  else locks[n].unlock();
}


int cb::SSL::passwordCallback(char *buf, int num, int rwflags, void *data) {
  string msg = "Enter ";
  if (rwflags) msg += "encryption";
  else msg += "decryption";
  msg += " password: ";

  string pass = SecurityUtilities::getpass(msg);

  strncpy(buf, pass.c_str(), pass.length());
  return pass.length();
}


void cb::SSL::flushErrors() {
  while (getError()) continue;
}


unsigned cb::SSL::getError() {
  return ERR_get_error();
}


unsigned cb::SSL::peekError() {
  return ERR_peek_error();
}


unsigned cb::SSL::peekLastError() {
  return ERR_peek_last_error();
}


string cb::SSL::getErrorStr(unsigned err) {
  string result;
  char buffer[120];

  while (true) {
    if (!err) err = ERR_get_error();
    if (!err) break;

    ERR_error_string(err, buffer);
    err = 0;

    if (!result.empty()) result += ", ";
    result += buffer;
  }

  return result;
}


const char *cb::SSL::getSSLErrorStr(int err) {
  switch (err) {
  case SSL_ERROR_NONE: return "ERROR_NONE";
  case SSL_ERROR_SSL: return "ERROR_SSL";
  case SSL_ERROR_WANT_READ: return "ERROR_WANT_READ";
  case SSL_ERROR_WANT_WRITE: return "ERROR_WANT_WRITE";
  case SSL_ERROR_WANT_X509_LOOKUP: return "ERROR_WANT_X509_LOOKUP";
  case SSL_ERROR_SYSCALL: return "ERROR_SYSCALL";
  case SSL_ERROR_ZERO_RETURN: return "ERROR_ZERO_RETURN";
  case SSL_ERROR_WANT_CONNECT: return "ERROR_WANT_CONNECT";
  case SSL_ERROR_WANT_ACCEPT: return "ERROR_WANT_ACCEPT";
  default: return "UNKNOWN SSL ERROR";
  }
}


int cb::SSL::createObject(const string &oid, const string &shortName,
                          const string &longName) {
  int nid = OBJ_create(oid.c_str(), shortName.c_str(), longName.c_str());

  if (nid == NID_undef)
    THROWS("Failed to create SSL object oid='" << oid << ", SN='" << shortName
           << "', LN='" << longName << "': " << getErrorStr());

  return nid;
}


int cb::SSL::findObject(const string &name) {
  int nid = OBJ_txt2nid(name.c_str());
  if (nid == NID_undef) THROWS("Unrecognized SSL object '" << name << "'");
  return nid;
}


void cb::SSL::init() {
  if (initialized) return;

  SSL_library_init();
  SSL_load_error_strings();
  ERR_load_crypto_strings();
  OpenSSL_add_all_algorithms();

  locks = new Mutex[CRYPTO_num_locks()];
  CRYPTO_set_id_callback(idCallback);
  CRYPTO_set_locking_callback(lockingCallback);

  initialized = true;
}


void cb::SSL::deinit() {
  if (!initialized) return;

  CRYPTO_set_id_callback(0);
  CRYPTO_set_locking_callback(0);

#if OPENSSL_VERSION_NUMBER < 0x1010000fL
  ERR_remove_state(0);
#endif
  ERR_free_strings();
  EVP_cleanup();
  CRYPTO_cleanup_all_ex_data();

  initialized = false;
}


void cb::SSL::infoCallback(int where, int ret) {
  if (where & SSL_CB_HANDSHAKE_START) handshakes++;
}


namespace {
  extern "C" void ssl_info_callback(const ::SSL *ssl, int where, int ret) {
    ((cb::SSL *)SSL_get_app_data(ssl))->infoCallback(where, ret);
  }
}


void cb::SSL::limitRenegotiation() {
  handshakes = 0;

  if (renegotiateLimited) return;
  renegotiateLimited = true;

  void *ptr = SSL_get_app_data(ssl);
  if (!ptr) SSL_set_app_data(ssl, (char *)this);
  else if (ptr != this)
    THROW("SSL app data already set.  Cannot renegotiate limit.");

  SSL_set_info_callback(ssl, ssl_info_callback);
}


void cb::SSL::checkHandshakes() {
  if (3 < handshakes)
    THROW("Potential Client-Initiated Renegotiation DOS attack detected");
}


bool cb::SSL::checkWants() {
  switch (state) {
  case WANTS_ACCEPT: accept(); break;
  case WANTS_CONNECT: connect(); break;
  default: break;
  }

  return state == PROCEED;
}

==> cbang-1.6.0.tar.gz: cbang-1.6.0/src/cbang/openssl/SSL.cpp <==
/******************************************************************************\

          This file is part of the C! library.  A.K.A the cbang library.

                Copyright (c) 2003-2019, Cauldron Development LLC
                   Copyright (c) 2003-2017, Stanford University
                               All rights reserved.

         The C! library is free software: you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public License
       as published by the Free Software Foundation, either version 2.1 of
               the License, or (at your option) any later version.

        The C! library is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                 Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
                 License along with the C! library.  If not, see
                         <http://www.gnu.org/licenses/>.

        In addition, BSD licensing may be granted on a case by case basis
        by written permission from at least one of the copyright holders.
           You may request written permission by emailing the authors.

                  For information regarding this software email:
                                 Joseph Coffland
                          joseph@cauldrondevelopment.com

\******************************************************************************/

#include "SSL.h"

#include "SecurityUtilities.h"
#include "BStream.h"
#include "Certificate.h"

#include <cbang/config.h>
#include <cbang/os/Mutex.h>
#include <cbang/os/Thread.h>

#include <cbang/log/Logger.h>

#ifdef HAVE_VALGRIND
#include <valgrind/memcheck.h>
#endif

#include <iostream>
#include <string.h>

// This avoids a conflict with OCSP_RESPONSE in wincrypt.h
#ifdef OCSP_RESPONSE
#undef OCSP_RESPONSE
#endif

#define OPENSSL_THREAD_DEFINES
#include <openssl/opensslconf.h>

#if !defined(OPENSSL_THREADS)
#error "OpenSSL must be compiled with thread support"
#endif

#include <openssl/crypto.h>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <openssl/opensslv.h>


using namespace std;
using namespace cb;

namespace {
  extern "C" void ssl_info_callback(const ::SSL *ssl, int where, int ret) {
    ((cb::SSL *)SSL_get_app_data(ssl))->infoCallback(where, ret);
  }
}



bool cb::SSL::initialized = false;
Mutex *cb::SSL::locks = 0;
unsigned cb::SSL::maxHandshakes = 3;


cb::SSL::SSL(_SSL *ssl) : ssl(ssl) {
  if (ssl) SSL_up_ref(ssl);
  init();
}


cb::SSL::SSL(const SSL &ssl) : SSL(ssl.ssl) {}


cb::SSL::SSL(SSL_CTX *ctx, BIO *bio) {
  init();
  ssl = SSL_new(ctx);
  if (!ssl) THROW("Failed to create new SSL");
  if (bio) setBIO(bio);
}


cb::SSL::~SSL() {if (ssl) SSL_free(ssl);}


void cb::SSL::setBIO(BIO *bio) {SSL_set_bio(ssl, bio, bio);}
bool cb::SSL::wantsRead()  const {return lastErr == SSL_ERROR_WANT_READ;}
bool cb::SSL::wantsWrite() const {return lastErr == SSL_ERROR_WANT_WRITE;}


void cb::SSL::setCipherList(const string &list) {
  if (!SSL_set_cipher_list(ssl, list.c_str()))
    THROW("Failed to set cipher list to: " << list);
}


string cb::SSL::getFullSSLErrorStr(int ret) const {
  string s = getSSLErrorStr(SSL_get_error(ssl, ret));
  if (peekError()) s += ", " + getErrorStr();
  return s;
}


bool cb::SSL::hasPeerCertificate() const {
  X509 *cert = SSL_get_peer_certificate(ssl);
  if (!cert) return false;
  X509_free(cert);
  return true;
}


void cb::SSL::verifyPeerCertificate() const {
  if (!hasPeerCertificate()) THROW("Peer did not present a certificate");

  if (SSL_get_verify_result(ssl) != X509_V_OK)
    THROW("Certificate does not verify");
}


SmartPointer<Certificate> cb::SSL::getPeerCertificate() const {
  if (SSL_get_verify_result(ssl) != X509_V_OK)
    THROW("Certificate does not verify");

  X509 *cert = SSL_get_peer_certificate(ssl);
  if (!cert) THROW("Peer did not present a certificate");

  return new Certificate(cert);
}


void cb::SSL::setTLSExtHostname(const string &hostname) {
  if (!SSL_set_tlsext_host_name(ssl, hostname.c_str()))
    THROW("Failed to set TLS host name extension to '" << hostname << "'");
}


void cb::SSL::setConnectState() {SSL_set_connect_state(ssl);}
void cb::SSL::setAcceptState()  {SSL_set_accept_state(ssl);}


void cb::SSL::connect() {
  LOG_DEBUG(5, "cb::SSL::connect()");

  lastErr = 0;
  int ret = SSL_connect(ssl);

  if (ret == -1) {
    lastErr = SSL_get_error(ssl, ret);
    if (lastErr == SSL_ERROR_WANT_READ || lastErr == SSL_ERROR_WANT_WRITE) {
      state = WANTS_CONNECT;
      return;
    }
  }

  state = PROCEED;

  if (ret != 1) THROW("SSL connect failed: " << getFullSSLErrorStr(ret));
}


void cb::SSL::accept() {
  LOG_DEBUG(5, "cb::SSL::accept()");

  // Limit renegotiation to prevent DOS attack
  handshakes = 0;
  void *ptr = SSL_get_app_data(ssl);
  if (!ptr) {
    SSL_set_app_data(ssl, (char *)this);
    SSL_set_info_callback(ssl, ssl_info_callback);
  }

  lastErr = 0;
  int ret = SSL_accept(ssl);

  if (ret == -1) {
    lastErr = SSL_get_error(ssl, ret);
    if (lastErr == SSL_ERROR_WANT_READ || lastErr == SSL_ERROR_WANT_WRITE) {
      state = WANTS_ACCEPT;
      return;
    }
  }

  state = PROCEED;

  // We can ignore WANT_READ etc. here because the Socket layer already
  // retries reads and writes up to the specified timeout.
  if (ret != 1) {
    string err = getFullSSLErrorStr(ret);
    LOG_DEBUG(5, "SSL accept failed: " << err);
    THROW("SSL accept failed: " << err);
  }
}


void cb::SSL::shutdown() {
  SSL_shutdown(ssl);
  LOG_DEBUG(5, "cb::SSL::shutdown() " << getErrorStr());

  flushErrors(); // Ignore errors
}


int cb::SSL::read(char *data, unsigned size) {
  LOG_DEBUG(5, "cb::SSL::read(" << size << ')');

  lastErr = 0;
  checkHandshakes();
  if (!checkWants() || !size) return 0;

  int ret = SSL_read(ssl, data, size);
  if (ret <= 0) {
    // Detect End of Stream
    if (SSL_get_shutdown(ssl) == SSL_RECEIVED_SHUTDOWN) return -1;

    lastErr = SSL_get_error(ssl, ret);
    if (lastErr == SSL_ERROR_WANT_READ || lastErr == SSL_ERROR_WANT_WRITE)
      return 0;

    string errMsg = getFullSSLErrorStr(ret);
    LOG_DEBUG(5, "cb::SSL::read() " << errMsg);
    THROW("SSL read failed: " << errMsg);
  }

  LOG_DEBUG(5, "cb::SSL::read()=" << ret);

#ifdef VALGRIND_MAKE_MEM_DEFINED
  (void)VALGRIND_MAKE_MEM_DEFINED(data, ret);
#endif // VALGRIND_MAKE_MEM_DEFINED

  return (unsigned)ret;
}


unsigned cb::SSL::write(const char *data, unsigned size) {
  LOG_DEBUG(5, "cb::SSL::write(" << size << ')');

  lastErr = 0;
  checkHandshakes();
  if (!checkWants() || !size) return 0;

  int ret = SSL_write(ssl, data, size);
  if (ret <= 0) {
    lastErr = SSL_get_error(ssl, ret);
    if (lastErr == SSL_ERROR_WANT_READ || lastErr == SSL_ERROR_WANT_WRITE)
      return 0;
    THROW("SSL write failed: " << getFullSSLErrorStr(ret));
  }

  LOG_DEBUG(5, "cb::SSL::write()=" << ret);
  return (unsigned)ret;
}


unsigned long cb::SSL::idCallback() {
  return (unsigned long)cb::Thread::self();
}


void cb::SSL::lockingCallback(int mode, int n, const char *file, int line) {
  if (mode & CRYPTO_LOCK) locks[n].lock();
  else locks[n].unlock();
}


int cb::SSL::passwordCallback(char *buf, int num, int rwflags, void *data) {
  string msg = "Enter ";
  if (rwflags) msg += "encryption";
  else msg += "decryption";
  msg += " password: ";

  string pass = SecurityUtilities::getpass(msg);

  strncpy(buf, pass.c_str(), pass.length());
  return pass.length();
}


void cb::SSL::flushErrors() {while (getError()) continue;}
unsigned cb::SSL::getError() {return ERR_get_error();}
unsigned cb::SSL::peekError() {return ERR_peek_error();}
unsigned cb::SSL::peekLastError() {return ERR_peek_last_error();}


string cb::SSL::getErrorStr(unsigned err) {
  string result;
  char buffer[120];

  while (true) {
    if (!err) err = ERR_get_error();
    if (!err) break;

    ERR_error_string(err, buffer);
    err = 0;

    if (!result.empty()) result += ", ";
    result += buffer;
  }

  return result;
}


const char *cb::SSL::getSSLErrorStr(int err) {
  switch (err) {
  case SSL_ERROR_NONE:             return "OK";
  case SSL_ERROR_SSL:              return "SSL";
  case SSL_ERROR_WANT_READ:        return "Want read";
  case SSL_ERROR_WANT_WRITE:       return "Want write";
  case SSL_ERROR_WANT_X509_LOOKUP: return "Want X509 lookup";
  case SSL_ERROR_SYSCALL:          return "Syscall";
  case SSL_ERROR_ZERO_RETURN:      return "Zero return";
  case SSL_ERROR_WANT_CONNECT:     return "Want connect";
  case SSL_ERROR_WANT_ACCEPT:      return "Want accept";
  default:                         return "Unknown";
  }
}


int cb::SSL::createObject(const string &oid, const string &shortName,
                          const string &longName) {
  int nid = OBJ_create(oid.c_str(), shortName.c_str(), longName.c_str());

  if (nid == NID_undef)
    THROW("Failed to create SSL object oid='" << oid << ", SN='" << shortName
           << "', LN='" << longName << "': " << getErrorStr());

  return nid;
}


int cb::SSL::findObject(const string &name) {
  int nid = OBJ_txt2nid(name.c_str());
  if (nid == NID_undef) THROW("Unrecognized SSL object '" << name << "'");
  return nid;
}


void cb::SSL::init() {
  if (initialized) return;

  SSL_library_init();
  SSL_load_error_strings();
  ERR_load_crypto_strings();
  OpenSSL_add_all_algorithms();

  locks = new Mutex[CRYPTO_num_locks()];
  CRYPTO_set_id_callback(idCallback);
  CRYPTO_set_locking_callback(lockingCallback);

  initialized = true;
}


void cb::SSL::deinit() {
  if (!initialized) return;

  CRYPTO_set_id_callback(0);
  CRYPTO_set_locking_callback(0);

#if OPENSSL_VERSION_NUMBER < 0x1010000fL
  ERR_remove_state(0);
#endif
  ERR_free_strings();
  EVP_cleanup();
  CRYPTO_cleanup_all_ex_data();

  initialized = false;
}


void cb::SSL::infoCallback(int where, int ret) {
  if (where & SSL_CB_HANDSHAKE_START) handshakes++;
}


void cb::SSL::checkHandshakes() {
  if (maxHandshakes < handshakes)
    THROW("Potential Client-Initiated Renegotiation DOS attack detected");
}


bool cb::SSL::checkWants() {
  switch (state) {
  case WANTS_ACCEPT:  accept();  break;
  case WANTS_CONNECT: connect(); break;
  default: break;
  }

  return state == PROCEED;
}
